# -*- coding: utf-8 -*-
import pygmsh

from helpers import compute_volume


def test():
    geom = pygmsh.built_in.Geometry()

    rectangle = geom.add_rectangle(0.0, 1.0, 0.0, 1.0, 0.0, 0.1)

    geom.add_raw_code("Recombine Surface {%s};" % rectangle.surface.id)

    ref = 1.0
    points, cells, _, _, _ = pygmsh.generate_mesh(geom, dim=2)

    assert abs(compute_volume(points, cells) - ref) < 1.0e-2 * ref

    return points, cells


if __name__ == "__main__":
    import meshio

    meshio.write_points_cells("quads.vtu", *test())



# -*- coding: utf-8 -*-
#
from __future__ import print_function

from .__about__ import __version__, __author__, __author_email__, __website__

from . import built_in
from . import opencascade
from .helpers import generate_mesh, get_gmsh_major_version, rotation_matrix

__all__ = [
    "built_in",
    "opencascade",
    "generate_mesh",
    "get_gmsh_major_version",
    "rotation_matrix",
    "__version__",
    "__author__",
    "__author_email__",
    "__website__",
]

try:
    import pipdate
except ImportError:
    pass
else:
    if pipdate.needs_checking(__name__):
        print(pipdate.check(__name__, __version__), end="")



"""Test translation for all dimensions."""
import numpy as np
import pygmsh
from helpers import compute_volume


def translation1d():
    """Translation of a line."""
    geom = pygmsh.built_in.Geometry()
    points = []
    for array in [[1, 0, 0], [0, 0, 0], [0, 1, 0]]:
        points.append(geom.add_point(array, 0.5))
    circle = geom.add_circle_arc(*points)
    points, _, _, _, _ = pygmsh.generate_mesh(geom)
    geom.translate(circle, [1.5, 0, 0])
    translated_points, _, _, _, _ = pygmsh.generate_mesh(geom)
    points[:, 0] = points[:, 0] + 1.5
    assert np.allclose(points, translated_points)


def translation2d():
    """Translation of a surface object."""
    geom = pygmsh.opencascade.Geometry(0.05, 0.05)
    disk = geom.add_disk([0, 0, 0], 1)
    disk2 = geom.add_disk([1.5, 0, 0], 1)
    geom.translate(disk, [1.5, 0, 0])
    geom.boolean_union([disk2, disk])

    points, cells, _, _, _ = pygmsh.generate_mesh(geom)
    surf = np.pi
    assert np.abs((compute_volume(points, cells) - surf) / surf) < 1e-3


def translation3d():
    """Translation of a volume object."""
    geom = pygmsh.opencascade.Geometry(0.05, 0.05)
    ball = geom.add_ball([0, 0, 0], 1)
    ball2 = geom.add_ball([1.5, 0, 0], 1)
    geom.translate(ball, [1.5, 0, 0])
    geom.boolean_union([ball2, ball])

    points, cells, _, _, _ = pygmsh.generate_mesh(geom)
    surf = 4 / 3 * np.pi
    assert isinstance(ball, pygmsh.opencascade.volume_base.VolumeBase)
    assert isinstance(ball, pygmsh.built_in.volume_base.VolumeBase)
    assert np.abs((compute_volume(points, cells) - surf) / surf) < 1e-3


if __name__ == "__main__":
    translation1d()
    translation2d()
    translation3d()



# -*- coding: utf-8 -*-
#


class Point(object):
    """
    Creates an elementary point.

    x : array-like[3]
        Give the three X, Y and Z coordinates of the
        point in the three-dimensional Euclidean space.
    lcar : float
        The prescribed mesh element size at this point.
    """

    _POINT_ID = 0

    def __init__(self, x, lcar=None):
        self.x = x
        self.lcar = lcar

        self.id = "p{}".format(Point._POINT_ID)
        Point._POINT_ID += 1

        # Points are always 3D in gmsh
        if lcar is not None:
            self.code = "\n".join(
                [
                    "{} = newp;".format(self.id),
                    "Point({}) = {{{!r}, {!r}, {!r}, {!r}}};".format(
                        self.id, x[0], x[1], x[2], lcar
                    ),
                ]
            )
        else:
            self.code = "\n".join(
                [
                    "{} = newp;".format(self.id),
                    "Point({}) = {{{!r}, {!r}, {!r}}};".format(
                        self.id, x[0], x[1], x[2]
                    ),
                ]
            )
        return



# -*- coding: utf-8 -*-
#
import codecs
import os

from setuptools import setup, find_packages

# https://packaging.python.org/single_source_version/
base_dir = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(base_dir, "pygmsh", "__about__.py"), "rb") as f:
    exec(f.read(), about)


def read(fname):
    return codecs.open(os.path.join(base_dir, fname), encoding="utf-8").read()


setup(
    name="pygmsh",
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    packages=find_packages(),
    description="Python frontend for Gmsh",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    url=about["__website__"],
    project_urls={"Documentation": "https://pygmsh.readthedocs.org/en/latest"},
    license=about["__license__"],
    platforms="any",
    install_requires=["meshio >=2.0, <3.0", "numpy >= 1.9"],
    extras_require={
        "all": ["pipdate >=0.3.0, <0.4.0"],
        "update": ["pipdate >=0.3.0, <0.4.0"],
    },
    classifiers=[
        about["__status__"],
        about["__license__"],
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Mathematics",
    ],
)



# -*- coding: utf-8 -*-
#
from .. import built_in


class VolumeBase(built_in.volume_base.VolumeBase):
    """
    Increments the Volume ID every time a new volume object
    is created. Inherits from built_in VolumeBase.
    """

    _ID = 0
    dimension = 3

    def __init__(self, is_list=False, id0=None):
        super(VolumeBase, self).__init__(id0=id0)

        self.is_list = is_list
        if is_list:
            self.id += "[]"
        return

    def char_length_code(self, char_length):
        if char_length is None:
            return []

        return [
            "pts_{}[] = PointsOf{{Volume{{{}}};}};".format(self.id, self.id),
            "Characteristic Length{{pts_{}[]}} = {};".format(self.id, char_length),
        ]



# -*- coding: utf-8 -*-
#

__version__ = "4.4.0"
__author__ = u"Nico Schlömer"
__author_email__ = "nico.schloemer@gmail.com"
__copyright__ = u"Copyright (c) 2013-2019, {} <{}>".format(__author__, __author_email__)
__website__ = "https://github.com/nschloe/pygmsh"
__license__ = "License :: OSI Approved :: MIT License"
__status__ = "Development Status :: 5 - Production/Stable"



# -*- coding: utf-8 -*-
#


class DefineConstant(object):
    def __init__(self, label, value, min_value, max_value, step=None, name=None):
        assert min_value <= value <= max_value

        self.label = label

        if name is None:
            name = "Parameters/{}".format(label)

        defined_constant = "{}, Min {}, Max {}".format(value, min_value, max_value)

        if step:
            defined_constant += ", Step {}".format(step)

        defined_constant += ', Name "{}"'.format(name)

        self.code = "DefineConstant[ {} = {{ {} }} ];".format(label, defined_constant)
        return

    # Need to overload repr so that the label will be formatted into gmsh code without any quotes
    def __repr__(self):
        return self.label



# -*- coding: utf-8 -*-
import pytest

import pygmsh

from helpers import compute_volume


@pytest.mark.skipif(pygmsh.get_gmsh_major_version() < 3, reason="requires Gmsh >= 3")
def test():
    geom = pygmsh.opencascade.Geometry()

    geom.add_box([0.0, 0.0, 0.0], [1, 2, 3], char_length=0.1)

    ref = 6
    points, cells, _, _, _ = pygmsh.generate_mesh(geom)
    assert abs(compute_volume(points, cells) - ref) < 1.0e-2 * ref
    return points, cells


if __name__ == "__main__":
    import meshio

    meshio.write_points_cells("opencascade_box.vtu", *test())



# -*- coding: utf-8 -*-
import pygmsh

from helpers import compute_volume


def test():
    geom = pygmsh.built_in.Geometry()

    xmin = geom.define_constant("xmin", 0.0, 0.0, 1.0)
    xmax = geom.define_constant("xmax", 1.0, 0.0, 1.0)
    ymin = geom.define_constant("ymin", 0.0, 0.0, 1.0)
    ymax = geom.define_constant("ymax", 1.0, 0.0, 1.0)
    geom.add_rectangle(xmin, xmax, ymin, ymax, 0.0, 0.1)

    ref = 1.0
    points, cells, _, _, _ = pygmsh.generate_mesh(geom, mesh_file_type="vtk")
    assert abs(compute_volume(points, cells) - ref) < 1.0e-2 * ref
    return points, cells


if __name__ == "__main__":
    import meshio

    meshio.write_points_cells("defined_constant_rectangle.vtu", *test())
